#ifndef MY_HISTOGRAM
#define MY_HISTOGRAM


#include "i3d/image3d.h"

/**
 * Computes histogram from the whole input image \e img.
 */
void ComputeHistogram(const i3d::Image3d<i3d::GRAY8>& img,
								int (&histogram)[256]);


/**
 * Computes histogram represented with my own custom link
 * from the whole input image \e img.
 *
 * Searches always from the begining of the list.
 *
 * No histogram is, in fact, saved. Instead, report
 * on complexity of the histogram computation is printed.
 */
void ComputeHistogram_MyList(const i3d::Image3d<i3d::GRAY8>& img);

/**
 * Computes histogram represented with my own custom link
 * from the whole input image \e img.
 *
 * Searches always from where it has added recently.
 *
 * No histogram is, in fact, saved. Instead, report
 * on complexity of the histogram computation is printed.
 */
void ComputeHistogram_MyListOpt(const i3d::Image3d<i3d::GRAY8>& img);

/**
 * Computes histogram represented with std map
 * from the whole input image \e img.
 *
 * Note: std map is balanced tree.
 *
 * No histogram is, in fact, saved. Instead, report
 * on complexity of the histogram computation is printed.
 */
void ComputeHistogram_StdMap(const i3d::Image3d<i3d::GRAY8>& img);

#endif
