#include <iostream>
#include "i3d/image3d.h"

//time measurement - according to POSIX (should run everywhere)
#include <time.h>


using namespace std;
using namespace i3d;

void ThresholdPOINTERS(Image3d<GRAY8> &img,const GRAY8 thres) {

	GRAY8 *p=img.GetFirstVoxelAddr();
	for (size_t i=0; i < img.GetImageSize(); ++i, ++p) {
		*p=(*p > thres)? 1 : 0;
	}
}

void ThresholdINDICES(Image3d<GRAY8> &img,const GRAY8 thres) {

	for (size_t i=0; i < img.GetImageSize(); ++i) {
		img.SetVoxel(i,
				(img.GetVoxel(i) > thres)? 1 : 0
				    );
	}
}

void ThresholdFORCYCLES(Image3d<GRAY8> &img,const GRAY8 thres) {

	for (size_t z=0; z < img.GetSizeZ(); ++z)
	  for (size_t y=0; y < img.GetSizeY(); ++y)
	    for (size_t x=0; x < img.GetSizeX(); ++x)
	    	{
			img.SetVoxel( x,y,z,
				(img.GetVoxel(x,y,z) > thres)? 1 : 0
				    );
		}
}

void ThresholdFORCYCLES_BAD(Image3d<GRAY8> &img,const GRAY8 thres) {

	for (size_t x=0; x < img.GetSizeX(); ++x)
	  for (size_t y=0; y < img.GetSizeY(); ++y)
	    for (size_t z=0; z < img.GetSizeZ(); ++z)
	    	{
			img.SetVoxel( x,y,z,
				(img.GetVoxel(x,y,z) > thres)? 1 : 0
				    );
		}
}

int main(void) {
	//opens and reads GRAY 8bit input image
	Image3d<GRAY8> img;
	img.ReadImageICS("images/cell04_3D_GRAY8.ics");

	//dry run - to fill caches and boost up CPU
	ThresholdPOINTERS(img,85);
	ThresholdPOINTERS(img,85);

	//time-measurement: time stamp before
	clock_t t1,t2,t3,t4,t5;
	t1 = clock();

	//the measured routine
	ThresholdPOINTERS(img,85);

	//time-measurement: time stamp after
	//...and before for the next measurement at the same time
	t2 = clock();

	ThresholdINDICES(img,85);

	t3 = clock();

	ThresholdFORCYCLES(img,85);

	t4 = clock();

	ThresholdFORCYCLES_BAD(img,85);

	t5 = clock();

	cout << "ThresholdPOINTERS() needed approx. " << (float)(t2-t1)/(float)CLOCKS_PER_SEC << " seconds\n";
	cout << "ThresholdINDICES() needed approx. " << (float)(t3-t2)/(float)CLOCKS_PER_SEC << " seconds\n";
	cout << "ThresholdFORCYCLES() needed approx. " << (float)(t4-t3)/(float)CLOCKS_PER_SEC << " seconds\n";
	cout << "ThresholdFORCYCLES_BAD() needed approx. " << (float)(t5-t4)/(float)CLOCKS_PER_SEC << " seconds\n";

	return(0);
}
